@@ -19,9 +19,9 @@
 //
 
 use crate::cache::ShardReaderCache;
+use crate::errors::ShardNotFoundError;
 use crate::grpc::update::{update_loop, UpdateParameters};
 use crate::settings::Settings;
-use crate::shards::errors::ShardNotFoundError;
 use crate::shards::reader::{ShardFileChunkIterator, ShardReader};
 use crate::telemetry::run_with_telemetry;
 use nucliadb_core::paragraphs::ParagraphIterator;
