@@ -20,7 +20,7 @@
 //! This module provides tools for managing shards
 
 // pub mod errors;
-mod indexes;
+pub mod indexes;
 pub mod metadata;
 pub mod shard_reader;
 pub mod shard_writer;
