@@ -21,6 +21,8 @@
 use std::io::Result;
 
 fn main() -> Result<()> {
+    std::env::set_var("PROTOC", protobuf_src::protoc());
+
     println!("cargo:rerun-if-changed=../knowledgebox.proto");
     println!("cargo:rerun-if-changed=../resources.proto");
     println!("cargo:rerun-if-changed=../noderesources.proto");
@@ -46,7 +48,7 @@ fn main() -> Result<()> {
         &["../../"],
     )?;
 
-    tonic_build::configure().build_server(true).out_dir("src").compile(
+    tonic_build::configure().build_server(true).emit_rerun_if_changed(false).out_dir("src").compile(
         &["nucliadb_protos/nodewriter.proto", "nucliadb_protos/nodereader.proto", "nucliadb_protos/replication.proto"],
         &["../../"],
     )?;
