@@ -30,6 +30,7 @@ use nucliadb_core::tracing::{self, *};
 use nucliadb_core::vectors::*;
 use nucliadb_procs::measure;
 use std::fmt::Debug;
+use std::path::Path;
 use std::time::Instant;
 
 impl<'a> SearchRequest for (usize, &'a VectorSearchRequest, Formula) {
@@ -60,11 +61,6 @@ impl Debug for VectorReaderService {
 }
 
 impl VectorReader for VectorReaderService {
-    fn update(&mut self) -> NodeResult<()> {
-        self.index.update()?;
-        Ok(())
-    }
-
     #[tracing::instrument(skip_all)]
     fn count(&self) -> NodeResult<usize> {
         debug!("Id for the vectorset is empty");
@@ -170,12 +166,12 @@ impl TryFrom<Neighbour> for DocumentScored {
 
 impl VectorReaderService {
     #[tracing::instrument(skip_all)]
-    pub fn start(config: &VectorConfig) -> NodeResult<Self> {
-        if !config.path.exists() {
-            return Err(node_error!("Invalid path {:?}", config.path));
+    pub fn open(path: &Path) -> NodeResult<Self> {
+        if !path.exists() {
+            return Err(node_error!("Invalid path {:?}", path));
         }
         Ok(VectorReaderService {
-            index: Reader::open(&config.path)?,
+            index: Reader::open(path)?,
         })
     }
 }
@@ -186,7 +182,7 @@ mod tests {
 
     use nucliadb_core::protos::resource::ResourceStatus;
     use nucliadb_core::protos::{
-        IndexParagraph, IndexParagraphs, Resource, ResourceId, VectorSentence, VectorSimilarity,
+        IndexParagraph, IndexParagraphs, Resource, ResourceId, VectorSentence, VectorSimilarity, VectorsetSentences,
     };
     use nucliadb_core::Channel;
     use tempfile::TempDir;
@@ -197,9 +193,10 @@ mod tests {
     #[test]
     fn test_key_prefix_search() {
         let dir = TempDir::new().unwrap();
+        let shard_path = dir.path().join("vectors");
         let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
-            path: dir.path().join("vectors"),
+            similarity: VectorSimilarity::Cosine,
+            path: shard_path.clone(),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
             normalize_vectors: false,
@@ -226,7 +223,13 @@ mod tests {
         let paragraph = IndexParagraph {
             start: 0,
             end: 0,
-            sentences,
+            sentences: sentences.clone(),
+            vectorsets_sentences: HashMap::from([(
+                "__default__".to_string(),
+                VectorsetSentences {
+                    sentences,
+                },
+            )]),
             field: "".to_string(),
             labels: vec!["1".to_string()],
             index: 3,
@@ -253,10 +256,10 @@ mod tests {
             ..Default::default()
         };
         // insert - delete - insert sequence
-        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let mut writer = VectorWriterService::create(vsc).unwrap();
         writer.set_resource(&resource).unwrap();
 
-        let reader = VectorReaderService::start(&vsc).unwrap();
+        let reader = VectorReaderService::open(&shard_path).unwrap();
         let mut request = VectorSearchRequest {
             id: "".to_string(),
             vector_set: "".to_string(),
@@ -280,9 +283,10 @@ mod tests {
     #[test]
     fn test_new_vector_reader() {
         let dir = TempDir::new().unwrap();
+        let shard_path = dir.path().join("vectors");
         let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
-            path: dir.path().join("vectors"),
+            similarity: VectorSimilarity::Cosine,
+            path: shard_path.clone(),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
             normalize_vectors: false,
@@ -309,7 +313,13 @@ mod tests {
         let paragraph = IndexParagraph {
             start: 0,
             end: 0,
-            sentences,
+            sentences: sentences.clone(),
+            vectorsets_sentences: HashMap::from([(
+                "__default__".to_string(),
+                VectorsetSentences {
+                    sentences,
+                },
+            )]),
             field: "".to_string(),
             labels: vec!["1".to_string()],
             index: 3,
@@ -336,10 +346,10 @@ mod tests {
             ..Default::default()
         };
         // insert - delete - insert sequence
-        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let mut writer = VectorWriterService::create(vsc).unwrap();
         let res = writer.set_resource(&resource);
         assert!(res.is_ok());
-        let reader = VectorReaderService::start(&vsc).unwrap();
+        let reader = VectorReaderService::open(&shard_path).unwrap();
         let request = VectorSearchRequest {
             id: "".to_string(),
             vector_set: "".to_string(),
@@ -402,9 +412,10 @@ mod tests {
     #[test]
     fn test_vectors_deduplication() {
         let dir = TempDir::new().unwrap();
+        let shard_path = dir.path().join("vectors");
         let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
-            path: dir.path().join("vectors"),
+            similarity: VectorSimilarity::Cosine,
+            path: shard_path.clone(),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
             normalize_vectors: false,
@@ -427,7 +438,13 @@ mod tests {
         let paragraph = IndexParagraph {
             start: 0,
             end: 0,
-            sentences,
+            sentences: sentences.clone(),
+            vectorsets_sentences: HashMap::from([(
+                "__default__".to_string(),
+                VectorsetSentences {
+                    sentences,
+                },
+            )]),
             field: "".to_string(),
             labels: vec!["1".to_string()],
             index: 3,
@@ -453,10 +470,10 @@ mod tests {
             shard_id: "DOC".to_string(),
             ..Default::default()
         };
-        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let mut writer = VectorWriterService::create(vsc).unwrap();
         let res = writer.set_resource(&resource);
         assert!(res.is_ok());
-        let reader = VectorReaderService::start(&vsc).unwrap();
+        let reader = VectorReaderService::open(&shard_path).unwrap();
         let request = VectorSearchRequest {
             id: "".to_string(),
             vector_set: "".to_string(),
