@@ -36,12 +36,13 @@ use nucliadb_core::{IndexFiles, RawReplicaState};
 use nucliadb_procs::measure;
 use std::collections::HashMap;
 use std::fmt::Debug;
+use std::path::{Path, PathBuf};
 use std::time::Instant;
 use std::time::SystemTime;
 
 pub struct VectorWriterService {
     index: Writer,
-    config: VectorConfig,
+    path: PathBuf,
 }
 
 impl Debug for VectorWriterService {
@@ -188,12 +189,12 @@ impl VectorWriter for VectorWriterService {
     }
 
     fn get_segment_ids(&self) -> NodeResult<Vec<String>> {
-        Ok(replication::get_segment_ids(&self.config.path)?)
+        Ok(replication::get_segment_ids(&self.path)?)
     }
 
     fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles> {
         // Should be called along with a lock at a higher level to be safe
-        let replica_state = replication::get_index_files(&self.config.path, "vectors", ignored_segment_ids)?;
+        let replica_state = replication::get_index_files(&self.path, "vectors", ignored_segment_ids)?;
 
         if replica_state.files.is_empty() {
             // exit with no changes
@@ -216,52 +217,31 @@ impl VectorWriterService {
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn start(config: &VectorConfig) -> NodeResult<Self> {
-        let path = std::path::Path::new(&config.path);
-        if !path.exists() {
-            match VectorWriterService::new(config) {
-                Err(e) if path.exists() => {
-                    std::fs::remove_dir(path)?;
-                    Err(e)
-                }
-                Err(e) => Err(e),
-                Ok(v) => Ok(v),
-            }
-        } else {
-            VectorWriterService::open(config)
-        }
-    }
-
-    #[tracing::instrument(skip_all)]
-    pub fn new(config: &VectorConfig) -> NodeResult<Self> {
+    pub fn create(config: VectorConfig) -> NodeResult<Self> {
         let path = std::path::Path::new(&config.path);
         if path.exists() {
             Err(node_error!("Shard does exist".to_string()))
         } else {
-            let Some(similarity) = config.similarity.map(|i| i.into()) else {
-                return Err(node_error!("A similarity must be specified, {:?}", config.similarity));
-            };
             let index_metadata = IndexMetadata {
-                similarity,
+                similarity: config.similarity.into(),
                 channel: config.channel,
                 normalize_vectors: config.normalize_vectors,
             };
             Ok(VectorWriterService {
-                index: Writer::new(path, index_metadata, config.shard_id.clone())?,
-                config: config.clone(),
+                index: Writer::new(path, index_metadata, config.shard_id)?,
+                path: path.to_path_buf(),
             })
         }
     }
 
     #[tracing::instrument(skip_all)]
-    pub fn open(config: &VectorConfig) -> NodeResult<Self> {
-        let path = std::path::Path::new(&config.path);
+    pub fn open(path: &Path, shard_id: String) -> NodeResult<Self> {
         if !path.exists() {
             Err(node_error!("Shard does not exist".to_string()))
         } else {
             Ok(VectorWriterService {
-                index: Writer::open(path, config.shard_id.clone())?,
-                config: config.clone(),
+                index: Writer::open(path, shard_id)?,
+                path: path.to_path_buf(),
             })
         }
     }
@@ -271,7 +251,7 @@ impl VectorWriterService {
 mod tests {
     use nucliadb_core::protos::resource::ResourceStatus;
     use nucliadb_core::protos::{
-        IndexParagraph, IndexParagraphs, Resource, ResourceId, VectorSentence, VectorSimilarity,
+        IndexParagraph, IndexParagraphs, Resource, ResourceId, VectorSentence, VectorSimilarity, VectorsetSentences,
     };
     use nucliadb_core::Channel;
     use std::collections::HashMap;
@@ -283,7 +263,7 @@ mod tests {
     fn test_new_vector_writer() {
         let dir = TempDir::new().unwrap();
         let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
+            similarity: VectorSimilarity::Cosine,
             path: dir.path().join("vectors"),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
@@ -310,7 +290,13 @@ mod tests {
         let paragraph = IndexParagraph {
             start: 0,
             end: 0,
-            sentences,
+            sentences: sentences.clone(),
+            vectorsets_sentences: HashMap::from([(
+                "__default__".to_string(),
+                VectorsetSentences {
+                    sentences,
+                },
+            )]),
             field: "".to_string(),
             labels: vec!["1".to_string(), "2".to_string(), "3".to_string()],
             index: 3,
@@ -337,7 +323,7 @@ mod tests {
             ..Default::default()
         };
         // insert - delete - insert sequence
-        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let mut writer = VectorWriterService::create(vsc).unwrap();
         let res = writer.set_resource(&resource);
         assert!(res.is_ok());
         let res = writer.delete_resource(&resource_id);
@@ -350,7 +336,7 @@ mod tests {
     fn test_get_segments() {
         let dir = TempDir::new().unwrap();
         let vsc = VectorConfig {
-            similarity: Some(VectorSimilarity::Cosine),
+            similarity: VectorSimilarity::Cosine,
             path: dir.path().join("vectors"),
             channel: Channel::EXPERIMENTAL,
             shard_id: "abc".into(),
@@ -373,7 +359,13 @@ mod tests {
         let paragraph = IndexParagraph {
             start: 0,
             end: 0,
-            sentences,
+            sentences: sentences.clone(),
+            vectorsets_sentences: HashMap::from([(
+                "__default__".to_string(),
+                VectorsetSentences {
+                    sentences,
+                },
+            )]),
             field: "".to_string(),
             labels: vec!["1".to_string(), "2".to_string(), "3".to_string()],
             index: 3,
@@ -400,7 +392,7 @@ mod tests {
             ..Default::default()
         };
         // insert - delete - insert sequence
-        let mut writer = VectorWriterService::start(&vsc).unwrap();
+        let mut writer = VectorWriterService::create(vsc).unwrap();
         let res = writer.set_resource(&resource);
         assert!(res.is_ok());
         let res = writer.delete_resource(&resource_id);
